@@ -14,6 +14,7 @@ use File::Temp;
 
 # XXX - make this an OO attribute
 our $VERBOSE = 0;
+our $NODIE = 0;
 
 =head1 NAME
 
@@ -21,11 +22,11 @@ Archive::Tar::Stream - pure perl IO-friendly tar file management
 
 =head1 VERSION
 
-Version 0.02
+Version 0.03
 
 =cut
 
-our $VERSION = '0.02';
+our $VERSION = '0.03';
 
 
 =head1 SYNOPSIS
@@ -83,6 +84,7 @@ Args:
    inpos     - initial offset in infh
    outpos    - initial offset in outfh
    safe_copy - boolean.
+   no_die    - boolean.
 
 Offsets are for informational purposes only, but can be
 useful if you are tracking offsets of items within your
@@ -103,12 +105,19 @@ sub new {
   my $class = shift;
   my %args = @_;
 
-  die "Useless to stream without a filehandle"
-    unless ($args{infh} or $args{outfh});
+  unless ($args{infh} or $args{outfh}) {
+    if($NODIE) {
+      warn "Useless to stream without a filehandle";
+      return undef;
+    } else {
+      die "Useless to stream without a filehandle";
+    }
+  }
 
   my $Self = bless {
     # defaults
     safe_copy => 1,
+    no_die => 0,
     inpos => 0,
     outpos => 0,
     %args
@@ -117,6 +126,23 @@ sub new {
   return $Self;
 }
 
+=head2 NoDie
+
+   $ts->NoDie(1);
+
+Toggle the "no_die" field mentioned above (stops you suffering a die() on errors encountered - it's your responsability to not corrupt your own files if you do this).
+
+=cut
+
+sub NoDie {
+  my $Self = shift;
+  if (@_) {
+    $Self->{no_die} = shift;
+    $NODIE=$Self->{no_die};
+  }
+  return $Self->{no_die};
+}
+
 =head2 SafeCopy
 
    $ts->SafeCopy(0);
@@ -319,7 +345,12 @@ sub StreamCopy {
       }
 
       else {
-        die "Bogus response $rc from callback\n";
+	if($NODIE) {
+          warn "Bogus response $rc from callback\n";
+	  return undef;
+        } else {
+          die "Bogus response $rc from callback\n";
+        }
       }
     }
     else {
@@ -359,7 +390,12 @@ sub ReadBlocks {
   my $Self = shift;
   my $nblocks = shift || 1;
   unless ($Self->{infh}) {
-    die "Attempt to read without input filehandle";
+    if($NODIE) {
+      warn "Attempt to read without input filehandle";
+      return undef;
+    } else {
+      die "Attempt to read without input filehandle";
+    }
   }
   my $bytes = BLOCKSIZE * $nblocks;
   my $buf;
@@ -369,7 +405,12 @@ sub ReadBlocks {
     unless ($n) {
       delete $Self->{infh};
       return if ($bytes == BLOCKSIZE * $nblocks); # nothing at EOF
-      die "Failed to read full block at $Self->{inpos}";
+      if($NODIE) {
+	warn "Failed to read full block at $Self->{inpos}";
+	return undef;
+      } else {
+	die "Failed to read full block at $Self->{inpos}";
+      }
     }
     $bytes -= $n;
     $Self->{inpos} += $n;
@@ -402,7 +443,12 @@ sub WriteBlocks {
   my $bytes = BLOCKSIZE * $nblocks;
 
   unless ($Self->{outfh}) {
-    die "Attempt to write without output filehandle";
+    if($NODIE) {
+      warn "Attempt to write without output filehandle";
+      return undef;
+    } else {
+      die "Attempt to write without output filehandle";
+    }
   }
   my $pos = $Self->{outpos};
 
@@ -418,7 +464,12 @@ sub WriteBlocks {
     my $n = syswrite($Self->{outfh}, $string, $bytes, (BLOCKSIZE * $nblocks) - $bytes);
     unless ($n) {
       delete $Self->{outfh};
-      die "Failed to write full block at $Self->{outpos}";
+      if($NODIE) {
+	warn "Failed to write full block at $Self->{outpos}";
+	return undef;
+      } else {
+	die "Failed to write full block at $Self->{outpos}";
+      }
     }
     $bytes -= $n;
     $Self->{outpos} += $n;
@@ -755,7 +806,12 @@ sub CopyFromFh {
     while ($thistime) {
       my $n = sysread($Fh, $buf, $thistime);
       unless ($n) {
-        die "Failed to read entire file, doh ($bytes remaining)!\n";
+        if($NODIE) {
+	  warn "Failed to read entire file, doh ($bytes remaining)!\n";
+	  return undef;
+	} else {
+	  die "Failed to read entire file, doh ($bytes remaining)!\n";
+	}
       }
       $thistime -= $n;
       $block .= $buf;
